@@ -305,6 +305,7 @@ void EffectsHandlerImpl::reconfigure()
     QFutureWatcher<KService::List> *watcher = new QFutureWatcher<KService::List>(this);
     connect(watcher, SIGNAL(finished()), this, SLOT(slotEffectsQueried()));
     watcher->setFuture(QtConcurrent::run(KServiceTypeTrader::self(), &KServiceTypeTrader::query, QString("KWin/Effect"), QString()));
+    watcher->waitForFinished(); // TODO: remove once KConfigGroup is thread safe, bug #321576
 }
 
 void EffectsHandlerImpl::slotEffectsQueried()
@@ -555,9 +556,9 @@ void EffectsHandlerImpl::slotClientAdded(Client *c)
 }
 
 void EffectsHandlerImpl::slotUnmanagedAdded(Unmanaged *u)
-{   // regardless, unmanaged windows are -yet?- not synced anyway
-    setupUnmanagedConnections(u);
-    emit windowAdded(u->effectWindow());
+{
+    // it's never initially ready but has synthetic 50ms delay
+    connect(u, SIGNAL(windowShown(KWin::Toplevel*)), SLOT(slotUnmanagedShown(KWin::Toplevel*)));
 }
 
 void EffectsHandlerImpl::slotClientShown(KWin::Toplevel *t)
@@ -569,6 +570,14 @@ void EffectsHandlerImpl::slotClientShown(KWin::Toplevel *t)
         emit windowAdded(c->effectWindow());
 }
 
+void EffectsHandlerImpl::slotUnmanagedShown(KWin::Toplevel *t)
+{   // regardless, unmanaged windows are -yet?- not synced anyway
+    Q_ASSERT(dynamic_cast<Unmanaged*>(t));
+    Unmanaged *u = static_cast<Unmanaged*>(t);
+    setupUnmanagedConnections(u);
+    emit windowAdded(u->effectWindow());
+}
+
 void EffectsHandlerImpl::slotDeletedRemoved(KWin::Deleted *d)
 {
     emit windowDeleted(d->effectWindow());
@@ -1036,8 +1045,10 @@ EffectWindowList EffectsHandlerImpl::stackingOrder() const
 {
     ToplevelList list = Workspace::self()->xStackingOrder();
     EffectWindowList ret;
-    foreach (Toplevel *w, list)
-        ret.append(effectWindow(w));
+    foreach (Toplevel *t, list) {
+        if (EffectWindow *w = effectWindow(t))
+            ret.append(w);
+    }
     return ret;
 }
 
