@@ -2,7 +2,7 @@
  KWin - the KDE window manager
  This file is part of the KDE project.
 
- Copyright (C) 2010 Martin Gräßlin <kde@martin-graesslin.com>
+ Copyright (C) 2010 Martin Gräßlin <mgraesslin@kde.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "startupfeedback.h"
 // Qt
-#include <QtCore/QSize>
+#include <QSize>
 #include <QtGui/QPainter>
 // KDE
 #include <KDE/KConfigGroup>
